var build, buildBrowser, buildNode, bundle, clean, coffee, commands, compile, docs, fs, glob, h, help, include, loadinfo, nodeunit, packageFile, path, publish, readline, run, scripts, stitch, test, uglify, ver, version, watch;
coffee = require('coffee-script');
fs = require('fs');
glob = require('glob');
h = require('./helpers');
loadinfo = require('./loadinfo');
nodeunit = require('nodeunit');
path = require('path');
readline = require('readline');
stitch = require('stitch');
uglify = require('uglify-js');
compile = function(info, file, sourcePath, targetPath) {
  var hookResult, source, src, target, ts, _base, _base2, _base3, _base4;
  file = fs.realpathSync(file);
  src = fs.readFileSync(file, 'utf8');
  source = file.substr(sourcePath.length);
  target = targetPath + source.replace(/.coffee$/, '.js');
  h.tryWithFile(file, 'tokenizing', function() {
    return src = coffee.tokens(src);
  });
  hookResult = typeof (_base = info.hookFns).tokenize === "function" ? _base.tokenize(target, src) : void 0;
  if (hookResult != null) {
    src = hookResult;
  }
  h.tryWithFile(file, 'parsing', function() {
    return src = coffee.nodes(src);
  });
  hookResult = typeof (_base2 = info.hookFns).parse === "function" ? _base2.parse(target, src) : void 0;
  if (hookResult != null) {
    src = hookResult;
  }
  h.tryWithFile(file, 'compiling', function() {
    return src = src.compile({
      bare: true
    });
  });
  hookResult = typeof (_base3 = info.hookFns).compile === "function" ? _base3.compile(target, src) : void 0;
  if (hookResult != null) {
    src = hookResult;
  }
  h.makeDir(path.dirname(target));
  fs.writeFileSync(target, src);
  if (typeof (_base4 = info.hookFns).write === "function") {
    _base4.write(target, src);
  }
  ts = (new Date()).toLocaleTimeString();
  source = sourcePath.substr(path.resolve('.').length + 1) + source;
  return console.log(ts + ' - compiled ' + source);
};
buildNode = function(info, watch, fn) {
  var compiled, count, sourcePath, targetPath, _, _base, _ref, _results;
  count = 0;
  for (_ in info.paths) {
    count++;
  }
  if (typeof (_base = info.hookFns).begin === "function") {
    _base.begin();
  }
  compiled = 0;
  _ref = info.paths;
  _results = [];
  for (sourcePath in _ref) {
    targetPath = _ref[sourcePath];
    _results.push((function(sourcePath, targetPath) {
      return h.withFiles(path.join(sourcePath, '**/*.coffee'), function(files) {
        var file, _base2, _i, _len;
        for (_i = 0, _len = files.length; _i < _len; _i++) {
          file = files[_i];
          compile(info, file, sourcePath, targetPath);
        }
        if (watch) {
          h.watchFiles(files, function(file) {
            try {
              return compile(info, file, sourcePath, targetPath);
            } catch (err) {
              return console.log(err.stack);
            }
          });
        }
        if (++compiled === count) {
          if (typeof (_base2 = info.hookFns).end === "function") {
            _base2.end();
          }
          return typeof fn === "function" ? fn() : void 0;
        }
      });
    })(sourcePath, targetPath));
  }
  return _results;
};
bundle = function(info) {
  var path, paths;
  paths = (function() {
    var _i, _len, _ref, _results;
    _ref = info.browser.paths;
    _results = [];
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      path = _ref[_i];
      _results.push(fs.realpathSync(path));
    }
    return _results;
  })();
  return stitch.createPackage({
    paths: paths
  }).compile(function(err, src) {
    var dir, fname, _base;
    if (err) {
      throw err;
    }
    h.makeDir('build');
    h.makeDir('build/' + info.version);
    dir = fs.realpathSync('build/' + info.version);
    fname = dir + '/' + info.browser.name;
    src = info.header + src + info.footer;
    fs.writeFileSync(fname + '.js', info.headerComment + src);
    fs.writeFileSync(fname + '.min.js', info.headerComment + uglify(src));
    try {
      fs.unlinkSync('build/edge');
    } catch (_e) {}
    fs.symlinkSync(dir + '/', 'build/edge');
    return typeof (_base = info.hookFns).bundle === "function" ? _base.bundle(fname + '.js') : void 0;
  });
};
buildBrowser = function(info, watch) {
  var paths, pth;
  bundle(info);
  if (watch) {
    paths = (function() {
      var _i, _len, _ref, _results;
      _ref = info.browser.paths;
      _results = [];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        pth = _ref[_i];
        _results.push(path.join(pth, '**/*.{coffee,js}'));
      }
      return _results;
    })();
    return h.watchFiles(glob.globSync("{" + (paths.join()) + "}"), function() {
      return bundle(info);
    });
  }
};
build = function(fn) {
  var info;
  info = loadinfo();
  return buildNode(info, false, function() {
    if (info.browser.enabled) {
      buildBrowser(info);
    }
    return typeof fn === "function" ? fn() : void 0;
  });
};
clean = function(target) {
  var dir, info, paths, pth, _, _i, _j, _len, _len2, _ref, _ref2;
  if (target == null) {
    target = 'all';
  }
  if (!h.knownTarget('clean', target, ['build', 'docs', 'all'])) {
    return;
  }
  info = loadinfo();
  paths = (function() {
    var _ref, _results;
    _ref = info.paths;
    _results = [];
    for (_ in _ref) {
      pth = _ref[_];
      _results.push(pth);
    }
    return _results;
  })();
  switch (target) {
    case 'build':
      _ref = paths.concat('build');
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        dir = _ref[_i];
        h.rmrf(dir);
      }
      break;
    case 'docs':
      h.rmrf('docs');
      break;
    case 'all':
      _ref2 = paths.concat('build', 'docs');
      for (_j = 0, _len2 = _ref2.length; _j < _len2; _j++) {
        dir = _ref2[_j];
        h.rmrf(dir);
      }
  }
};
docs = function() {
  var info, paths, pth, _, _ref;
  info = loadinfo();
  paths = (function() {
    var _results;
    _results = [];
    for (pth in info.paths) {
      _results.push(path.join(pth, '**/*.coffee'));
    }
    return _results;
  })();
  _ref = info.hooks;
  for (_ in _ref) {
    pth = _ref[_];
    if (pth) {
      paths.push(pth + '.{coffee,js}');
    }
  }
  return h.withFiles("{" + (paths.join()) + "}", function(files) {
    return h.tryExec('docco', '"' + files.join('" "') + '"');
  });
};
help = function() {
  var command, name, _results;
  _results = [];
  for (name in commands) {
    command = commands[name];
    _results.push(console.log("beans " + name + "\t" + command[1]));
  }
  return _results;
};
include = function() {
  var fn, info, sources, target, _ref;
  info = loadinfo();
  _ref = info.browser.include;
  for (target in _ref) {
    sources = _ref[target];
    fn = function(contents) {
      target = path.resolve(target);
      h.makeDir(path.dirname(target));
      return h.cat(target, contents);
    };
    if (!Array.isArray(sources)) {
      sources = [sources];
    }
    h.fetch(fn, sources);
  }
};
publish = function() {
  return build(function() {
    return h.tryExec('npm', 'publish');
  });
};
scripts = function() {
  return h.ask('This will modify package.json. Proceed?', ['y', 'n'], function(answer) {
    var deps, package, script, _i, _len, _ref;
    if (answer === 'y') {
      package = JSON.parse(fs.readFileSync('package.json'));
      deps = package.devDependencies || (package.devDependencies = {});
      deps.beans || (deps.beans = '~' + ver);
      scripts = package.scripts || (package.scripts = {});
      _ref = ['build', 'clean', 'docs', 'include', 'test', 'watch'];
      for (_i = 0, _len = _ref.length; _i < _len; _i++) {
        script = _ref[_i];
        scripts[script] = 'beans ' + script;
      }
      scripts.prepublish = 'beans build';
      return fs.writeFileSync('package.json', JSON.stringify(package, null, 2) + "\n");
    }
  });
};
test = function() {
  return build(function() {
    return h.withFiles('test/**/*.test.coffee', function(files) {
      return nodeunit.reporters["default"].run(files);
    });
  });
};
packageFile = path.resolve(__dirname, '../package.json');
ver = JSON.parse(fs.readFileSync(packageFile)).version;
version = function() {
  return console.log('Beans ' + ver);
};
watch = function() {
  var info;
  info = loadinfo();
  return buildNode(info, true, function() {
    if (info.browser.enabled) {
      return buildBrowser(info, true);
    }
  });
};
commands = {
  build: [build, 'Compile CoffeScript source for enabled targets.'],
  clean: [clean, 'Remove generated directories and tidy things up.'],
  docs: [docs, 'Generate documentation files using Docco.'],
  help: [help, 'Display help (this text).'],
  include: [include, 'Fetch local and remote includes.'],
  publish: [publish, 'Build everything and run npm publish.'],
  scripts: [scripts, 'Register beans in package.json scripts.'],
  test: [test, 'Build everything and run tests using nodeunit.'],
  version: [version, 'Display current Beans version.'],
  watch: [watch, 'Build everything once, then watch for changes.']
};
run = function() {
  var args, _ref;
  args = process.argv.slice(2);
  if (args.length === 0) {
    version();
    return help();
  } else if (commands[args[0]] != null) {
    return (_ref = commands[args[0]])[0].apply(_ref, args.slice(1));
  } else {
    return console.log("Don't know how to \"" + args[0] + "\".");
  }
};
module.exports = {
  commands: commands,
  run: run,
  version: ver
};