var exec, exports, fs, glob, ifInstalled, path, request, rimraf, which;
fs = require('fs');
glob = require('glob');
path = require('path');
request = require('request');
rimraf = require('rimraf');
which = require('which');
exec = require('child_process').exec;
module.exports = exports = {};
ifInstalled = function(executable, fn) {
  return which(executable, function(err) {
    if (!err) {
      return fn();
    }
    return console.log("This task needs \"" + executable + "\" to be installed and in PATH.");
  });
};
exports.tryExec = function(executable, args, fn) {
  return ifInstalled(executable, function() {
    var proc;
    proc = exec(executable + ' ' + args, function(err) {
      if (err) {
        throw err;
      }
      return typeof fn === "function" ? fn() : void 0;
    });
    return proc.stdout.on('data', function(data) {
      return process.stdout.write(data.toString());
    });
  });
};
exports.rmrf = function(path, fn) {
  return rimraf(path, function(err) {
    if (err) {
      throw err;
    }
    return typeof fn === "function" ? fn() : void 0;
  });
};
exports.makeDir = function(dir) {
  var missing, _i, _len, _results;
  missing = [];
  dir = path.resolve(dir);
  while (!path.existsSync(dir)) {
    missing.unshift(dir);
    dir = path.dirname(dir);
  }
  _results = [];
  for (_i = 0, _len = missing.length; _i < _len; _i++) {
    dir = missing[_i];
    _results.push(fs.mkdirSync(dir, 0755));
  }
  return _results;
};
exports.cat = function(target, strings) {
  var fd, s, _i, _len;
  fd = fs.openSync(target, 'w');
  for (_i = 0, _len = strings.length; _i < _len; _i++) {
    s = strings[_i];
    fs.writeSync(fd, s, null);
    if (s.substr(-1) !== "\n") {
      fs.writeSync(fd, "\n", null);
    }
  }
  return fs.closeSync(fd);
};
exports.fetch = function(fn, paths) {
  var contents, current, i, path, step, total, _len, _results;
  contents = [];
  current = 0;
  total = paths.length;
  step = function() {
    if (++current === total) {
      return fn(contents);
    }
  };
  _results = [];
  for (i = 0, _len = paths.length; i < _len; i++) {
    path = paths[i];
    _results.push((function(path, i) {
      if (/^[a-z]+:\/\//.test(path)) {
        console.log("fetch: " + path);
        return request(path, function(err, response, body) {
          if (err) {
            throw err;
          }
          contents[i] = body;
          return step();
        });
      } else {
        contents[i] = fs.readFileSync(path, 'utf8');
        return step();
      }
    })(path, i));
  }
  return _results;
};
exports.withFiles = function(pattern, fn) {
  var files;
  files = glob.globSync(pattern);
  if (files.length > 0) {
    return fn(files);
  }
};
exports.ask = function(question, answers, fn) {
  var stdin, stdout;
  stdin = process.stdin, stdout = process.stdout;
  stdout.write(question + ' (' + answers.join('/') + ') ');
  stdin.resume();
  return stdin.on('data', function(answer) {
    answer = answer.toString().trim();
    if (answers.indexOf(answer) !== -1) {
      stdin.pause();
      return fn(answer);
    } else {
      return stdout.write('Please answer with one of: (' + answers.join('/') + ') ');
    }
  });
};
exports.knownTarget = function(command, target, targets) {
  if (targets.indexOf(target) === -1) {
    console.log("Unknown " + command + " target \"" + target + "\".");
    console.log("Try one of: " + (targets.join(', ')) + ".");
    return false;
  }
  return true;
};
exports.watchFiles = function(files, fn) {
  var file, _i, _len, _results;
  _results = [];
  for (_i = 0, _len = files.length; _i < _len; _i++) {
    file = files[_i];
    _results.push((function(file) {
      return fs.watchFile(file, {
        persistent: true,
        interval: 500
      }, function(curr, prev) {
        if (curr.mtime.getTime() !== prev.mtime.getTime()) {
          return fn(file);
        }
      });
    })(file));
  }
  return _results;
};
exports.tryWithFile = function(file, event, fn) {
  try {
    return fn();
  } catch (err) {
    err.message = "When " + event + " " + file + ", " + err.message;
    throw err;
  }
};